//! Macros for testing `nom` parsers
//!
//! Often when I'm testing `nom` parsers, I end up defining a lot of little
//! macros like this, so I thought I would bundle them all up into a crate
//! so I didn't have to define them over and over
//!
#![deny(missing_docs)] // <- this doesn't appear to check macros?

#[macro_export]
/// This macro checks to make sure that the IResult it is
/// passed is `Done`
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # #[macro_use] extern crate nom;
/// # fn main() {
/// named!(abcd<&str, &str>, tag_s!("abcd"));
/// let r = abcd("abcd");
/// assert_done!(r);
/// # }
/// ```
macro_rules! assert_done {
    ($e:expr) => {
        {
            use nom::IResult;
            if let IResult::Done(_, _) = $e {
                assert!(true);
            } else {
                panic!("parser did not complete");
            }
        }
    }
}

#[macro_export]
/// Internal only, really, and not even necessary :P
macro_rules! assert_empty {
    ($v:expr) => {
        assert!($v.is_empty())
    }
}

#[macro_export]
/// This does the same thing as `assert_done!`, except that
/// this also asserts that the input slice is empty
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::IResult;
/// # fn main() {
/// let r: IResult<&str, &str> = IResult::Done("", "efgh");
/// assert_finished!(r);
/// # }
/// ```
macro_rules! assert_finished {
    ($e:expr) => {
        {
            use nom::IResult;
            if let IResult::Done(i, _) = $e {
                assert_empty!(i);
            } else {
                panic!("parser did not complete");
            }
        }
    }
}

#[macro_export]
/// This checks that the `IResult` is `Done`, and lets you
/// check that the value returned as the `O` type of the 
/// `IResult` is equal to the second parameter
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # fn main() {
/// # use nom::IResult;
/// let r: IResult<&[u8], &[u8]> = IResult::Done(b"abcd", b"efgh");
/// assert_done_and_eq!(r, b"efgh");
/// # }
macro_rules! assert_done_and_eq {
    ($e:expr, $a:expr) => {
        {
            use nom::IResult;
            if let IResult::Done(_, o) = $e {
                assert_eq!(o, $a);
            } else {
                panic!("parser did not complete");
            }
        }
    }
}

#[macro_export]
/// Same as `assert_done_and_eq!`, but asserts that
/// the input slice is empty
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::IResult;
/// # fn main() {
/// let r: IResult<&str, &str> = IResult::Done("", "sup");
/// assert_finished_and_eq!(r, "sup");
/// # }
/// ```
macro_rules! assert_finished_and_eq {
    ($r:expr, $o:expr) => {
        {
            use nom::IResult;
            if let IResult::Done(i, o) = $r {
                assert_empty!(i);
                assert_eq!(o, $o);
            } else {
                panic!("parser did not complete");
            }
        }
    }
}

#[macro_export]
/// This asserts that the `IResult` is an `Error`
/// 
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::{IResult, Err, ErrorKind};
/// # fn main() {
/// let r: IResult<&[u8], &[u8]> = IResult::Error(Err::Code(ErrorKind::Custom(1)));
/// assert_error!(r);
/// # }
macro_rules! assert_error {
    ($e:expr) => {
        {
            use nom::IResult;
            if let IResult::Error(_) = $e {
                assert!(true);
            } else {
                panic!("parser did not error");
            }
        }
    }
}

#[macro_export]
/// This asserts that the `IResult` is an `Error` and that the error
/// is what is expected
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::{IResult, Err, ErrorKind};
/// # fn main() {
/// let r: IResult<&[u8], &[u8]> = IResult::Error(Err::Code(ErrorKind::Custom(1)));
/// assert_error_and_eq!(r, Err::Code(ErrorKind::Custom(1)));
/// # }
macro_rules! assert_error_and_eq {
    ($r:expr, $err:expr) => {
        {
            use nom::IResult;
            if let IResult::Error(err) = $r {
                assert_eq!(err, $err);
            } else {
                panic!("parser did not error");
            }
        }
    }
}

#[macro_export]
/// This asserts that the `IResult` is an `Incomplete`
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::{IResult, Needed};
/// # fn main() {
/// let r: IResult<&[u8], &[u8]> = IResult::Incomplete(Needed::Size(1));
/// assert_needed!(r);
/// # }
/// ```
macro_rules! assert_needed {
    ($e:expr) => {
        {
            use nom::IResult;
            if let IResult::Incomplete(_) = $e {
                assert!(true);
            } else {
                panic!("parser is not incomplete");
            }
        }
    }
}

#[macro_export]
/// This lets the user specify how much input the parser should need
///
/// # Examples
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::{IResult, Needed};
/// # fn main() {
/// let r: IResult<&[u8], &[u8]> = IResult::Incomplete(Needed::Unknown);
/// assert_needs!(r, ?);
/// # }
/// ```
///
/// ```
/// # #[macro_use] extern crate nom_test_helpers;
/// # extern crate nom;
/// # use nom::{IResult, Needed};
/// # fn main() {
/// let r: IResult<&[u8], &[u8]> = IResult::Incomplete(Needed::Size(2));
/// assert_needs!(r, 2);
/// # }
/// ```
macro_rules! assert_needs {
    ($e:expr, ? ) => {
        {
            use nom::{IResult, Needed};
            if let IResult::Incomplete(e) = $e {
                if let Needed::Unknown = e {
                    assert!(true);
                } else {
                    panic!("parser is incomplete, but Needed is known");
                }
            } else {
                panic!("parser is not incomplete");
            }
        }
    };

    ($e:expr, $i:expr) => {
        {
            use nom::{IResult, Needed};
            if let IResult::Incomplete(e) = $e {
                if let Needed::Size(i) = e {
                    assert_eq!($i, i);
                } else {
                    panic!("parser is incomplete, but Needed is unknown");
                }
            } else {
                panic!("parser is not incomplete");
            }
        }
    }
}
